#!/usr/bin/python
# -*- coding:utf-8 -*-
#
#   Copyright 2008-2009 Le Coz Florent <louizatakk@fedoraproject.org>
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the WTF Public License as published by
#   Sam Hocevar.
#
#   DO WHAT THE FUCK YOU WANT TO PUBLIC LICENSE
#   Version 2, December 2004
#   DO WHAT THE FUCK YOU WANT TO PUBLIC LICENSE
#   TERMS AND CONDITIONS FOR COPYING, DISTRIBUTION AND MODIFICATION
#   0. You just DO WHAT THE FUCK YOU WANT TO.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#

import pgu
import random
import base64
import gtk, os, sys
import ConfigParser
import xmpp

from time import sleep
from game import *
from connection import *
from deckcreator import *

VERSION = 0.4
NAME = 'ludimagia'

if os.name == 'posix':
    CONFIG_PATH = os.environ['HOME']+'/.config/'+NAME+'/'
elif os.name == 'nt':
    CONFIG_PATH = 'config/'
try:
    os.makedirs(CONFIG_PATH)
except OSError:
    pass

CARDS_DIR = 'cards/'
CONFIG_FILE = CONFIG_PATH+NAME+'.conf'

class MainMenu(gtk.Window):
    '''Main menu that let the player choose what to do
    Create and/or modify a deck (deckcreator)
    Play over the internet against someone else
    Change the user preferences'''

    def __init__(self):
        gtk.Window.__init__(self)
        self.ok = False
        self.set_title('Ludimagia - Main Menu v%s' % (VERSION))
        self.set_icon_from_file('images/logo.png')
        self.set_default_size(400, 300)
        self.set_border_width(0)
        self.connect("destroy", gtk.main_quit)
        self.vbox = gtk.VBox()

        self.deck_creator_but = gtk.Button(label='Create or modify decks')
        self.play_but = gtk.Button(label='Play ludimagia')
        self.pref_but = gtk.Button(label='Preferences')
        self.deck_creator_but.connect('clicked', self.open_deck_creator)
        self.play_but.connect('clicked', self.open_game)
        self.pref_but.connect('clicked', self.open_pref)
        self.vbox.pack_start(self.play_but, True, True, 6)
        self.vbox.pack_start(self.pref_but, True, True, 6)
        self.vbox.pack_start(self.deck_creator_but, True, True, 6)
        self.add(self.vbox)
        self.config = ConfigParser.ConfigParser()
        self.config.read(CONFIG_FILE)
        self.show_all()

    def open_game(self, event=None):
        self.destroy()
        self.ok = True
        return

    def open_deck_creator(self, event=None):
        self.destroy()
        dc = DeckCreator(True)
        gtk.main()
        return

    def open_pref(self, event=None):
        pref = Preferences(self.config)
        return

class Preferences(gtk.Window):
    '''Preferences window'''
    def __init__(self, config):
        gtk.Window.__init__(self)
        self.config = config

        try: prox_user = self.config.get('DEFAULT', 'prox_user')
        except ConfigParser.NoOptionError: prox_user = ''

        try: prox_passwd = base64.b64decode(self.config.get('DEFAULT', 'prox_passwd'))
        except ConfigParser.NoOptionError: prox_passwd = ''

        try: prox_port = self.config.get('DEFAULT', 'prox_port')
        except ConfigParser.NoOptionError: prox_port = ''

        try: prox_host = self.config.get('DEFAULT', 'prox_host')
        except ConfigParser.NoOptionError: prox_host = ''

        try: server = self.config.get('DEFAULT', 'server')
        except ConfigParser.NoOptionError: server = ''

        try: port = self.config.get('DEFAULT', 'port')
        except ConfigParser.NoOptionError: port = ''

        box = gtk.VBox()

        prox_userbox = gtk.HBox()
        self.prox_userentry = gtk.Entry()
        self.prox_userentry.set_text(prox_user)
        prox_userbox.pack_start(gtk.Label('Proxy Username'))
        prox_userbox.pack_start(self.prox_userentry)

        prox_passwdbox = gtk.HBox()
        self.prox_passwdentry = gtk.Entry()
        self.prox_passwdentry.set_text(prox_passwd)
        self.prox_passwdentry.set_visibility(False)
        prox_passwdbox.pack_start(gtk.Label('Proxy Password'))
        prox_passwdbox.pack_start(self.prox_passwdentry)

        prox_portbox = gtk.HBox()
        self.prox_portentry = gtk.Entry()
        self.prox_portentry.set_text(prox_port)
        prox_portbox.pack_start(gtk.Label('Proxy Port'))
        prox_portbox.pack_start(self.prox_portentry)

        prox_hostbox = gtk.HBox()
        self.prox_hostentry = gtk.Entry()
        self.prox_hostentry.set_text(prox_host)
        prox_hostbox.pack_start(gtk.Label('Proxy Host'))
        prox_hostbox.pack_start(self.prox_hostentry)

        serverbox = gtk.HBox()
        self.serverentry = gtk.Entry()
        self.serverentry.set_text(server)
        serverbox.pack_start(gtk.Label('Server'))
        serverbox.pack_start(self.serverentry)

        portbox = gtk.HBox()
        self.portentry = gtk.Entry()
        self.portentry.set_text(port)
        portbox.pack_start(gtk.Label('Port'))
        portbox.pack_start(self.portentry)

        box.pack_start(serverbox)
        box.pack_start(portbox)
        box.pack_start(prox_userbox)
        box.pack_start(prox_passwdbox)
        box.pack_start(prox_hostbox)
        box.pack_start(prox_portbox)

        but_ok = gtk.Button(stock=gtk.STOCK_OK)
        but_ok.connect('clicked', self.save_preferences)
        but_cancel = gtk.Button(stock=gtk.STOCK_CANCEL)
        but_cancel.connect('clicked', self.exit_preferences)
        box.pack_start(but_ok)
        box.pack_start(but_cancel)

        self.add(box)
        self.show_all()

    def save_preferences(self, event=None):
        self.config.set('DEFAULT', 'prox_user', self.prox_userentry.get_text())
        self.config.set('DEFAULT', 'prox_port', self.prox_portentry.get_text())
        self.config.set('DEFAULT', 'prox_passwd', base64.b64encode(self.prox_passwdentry.get_text()))
        self.config.set('DEFAULT', 'prox_host', self.prox_hostentry.get_text())
        self.config.set('DEFAULT', 'server', self.serverentry.get_text())
        self.config.set('DEFAULT', 'port', self.portentry.get_text())
        self.config.write(open(CONFIG_FILE, 'w'))
        self.destroy()

    def exit_preferences(self, event=None):
        self.destroy()

class Game_Startup(gtk.Window):
    '''The window where the player enters the options (the deck, its JID,
    the opponent\'s JID, etc), then starts the game'''

    def __init__(self):
        gtk.Window.__init__(self)
        self.ok = False
        self.set_title('Ludimagia v%s' % (VERSION))
        self.set_icon_from_file('images/logo.png')
        self.set_default_size(30, 30)
        self.set_border_width(0)
        self.connect("destroy", gtk.main_quit)
        self.vbox = gtk.VBox()

        self.config = ConfigParser.ConfigParser()
        self.config.read(CONFIG_FILE)

        self.show_all()
        self.list_cards = get_list_all_cards()

        jidbox = gtk.HBox()
        try:
            jid = self.config.get('DEFAULT', 'jid')
        except ConfigParser.NoOptionError:
            jid = ''
        jidbox.pack_start(gtk.Label('Your JID: '))
        self.jid = gtk.Entry()
        self.jid.set_text(jid)
        jidbox.pack_start(self.jid)
        self.vbox.pack_start(jidbox)

        passwdbox = gtk.HBox()
        passwdbox.pack_start(gtk.Label('Password: '))
        self.passwd = gtk.Entry()
        self.passwd.set_visibility(False)
        passwdbox.pack_start(self.passwd)
        self.vbox.pack_start(passwdbox)

        self.checkbutton =  gtk.CheckButton(label='Remember password')
        try:
            passwd = self.config.get('DEFAULT', 'password')
        except ConfigParser.NoOptionError:
            passwd = ''
        if passwd != '':
            passwd = base64.b64decode(passwd)
            self.checkbutton.set_active(True)
            self.passwd.set_text(passwd)
        else:
            self.checkbutton.set_active(False)
        self.vbox.pack_start(self.checkbutton)
        opjidbox = gtk.HBox()
        opjidbox.pack_start(gtk.Label('Your opponent\'s JID: '))
        liststore = gtk.ListStore(str)
        try:
            self.jids = self.config.get('DEFAULT', 'opjids')
        except ConfigParser.NoOptionError:
            self.jids = ''
        for jid in self.jids.split(':'):
             liststore.insert(0, [jid])
        self.opjid = gtk.ComboBoxEntry(liststore)
        opjidbox.pack_start(self.opjid)
        self.vbox.pack_start(opjidbox)
        self.add(self.vbox)

        deckbox = gtk.HBox()
        deckbox.pack_start(gtk.Label('Deck: '))
        self.deck = gtk.Entry()
        self.deck.set_editable(False)
        deckbox.pack_start(self.deck)
        deckbut = gtk.Button('Choose')
        deckbut.connect('clicked', self.choose_deck)
        self.checkbut = gtk.Button('Check validity')
        self.checkbut.connect('clicked', self.check_deck)
        deckbox.pack_start(deckbut)
        deckbox.pack_start(self.checkbut)
        self.vbox.pack_start(deckbox)

        self.start_but = gtk.Button('Go !', gtk.STOCK_OK)
        self.vbox.pack_start(self.start_but)
        self.start_but.connect('clicked', self.start_game)
        self.show_all()

    def start_game(self, event=None):
        ## starts only if all informations are valid.
        if self.checkbut.get_label() == 'The deck is valid!' and self.opjid.get_active() != '' and self.jid.get_text() != '':
            ## save the preferences
            if self.opjid.get_child().get_text() not in self.jids.split(':'):
                self.jids += ':'+self.opjid.get_child().get_text()
                self.config.set('DEFAULT', 'opjids', self.jids)
            self.config.set('DEFAULT', 'jid', self.jid.get_text())
            if self.checkbutton.get_active():
                self.config.set('DEFAULT', 'password', base64.b64encode(self.passwd.get_text()))
            else:
                self.config.set('DEFAULT', 'password', '')
            self.config.write(open(CONFIG_FILE, 'w'))
        else:
            return
        ## connect to jabber
        self.jabber = Jabber(self.jid.get_text(),
                        self.passwd.get_text(),
                        self.opjid.get_child().get_text(),
                        dict(self.config.items('DEFAULT')),
                        self.get_deck(self.deck.get_text()))
        self.jabber.start()
        self.ok = True
        self.destroy()
        return
        # start_new_thread(jabber_connection, (self.jid.get_text(), self.passwd.get_text(), self.opjid.get_child().get_text()))

    def get_deck(self, deck):
        '''returns a list of card names (and only the names) contained in the deck file'''
        fd = open(deck, 'r')
        lines = fd.readlines()
        deck_ret = []
        for line in lines:
            if line.startswith('//') or line.startswith('SB:') or line.startswith(' -') or len(line.strip()) == 0:
                continue
            else:
                li = line.strip().split(' ')
                if li[1].startswith('['):
                    name = ' '.join(li[2:])
                else:
                    name = ' '.join(li[1:])
                if name.find('(') != -1:
                    name = name[:name.find('(')-1]
                if name in deck_ret:
                    continue
                nb = li[0]
                for i in xrange(int(nb)):
                    deck_ret.append(name)
        fd.close()
        random.shuffle(deck_ret)
        return deck_ret

    def check_deck(self, event=None):
        '''check deck validity'''
        self.checkbut.set_label('Checking...')
        self.ldeck = []
        txt = self.deck.get_text()
        if txt == '':
            self.checkbut.set_label('The deck does not exist')
            return
        try:
            fd = open(txt, 'r')
        except:
            self.checkbut.set_label('The deck does not exist')
            return False
        lines = fd.readlines()
        for line in lines: ## check file format
            if line.startswith('//') or line.startswith('SB:') or line.startswith(' -') or len(line.strip()) == 0:
                continue
            elif line.startswith('    ') == False:
                self.checkbut.set_label('The deck is NOT valid!\n Wrong format')
                fd.close()
                return False
        for line in lines: ## check that all cards are known (from the all_card_list)
            if line.startswith('//') or line.startswith('SB:') or line.startswith(' -') or len(line.strip()) == 0:
                continue
            else:
                li = line.strip().split(' ')
                if li[1].startswith('['):
                    name = ' '.join(li[2:])
                else:
                    name = ' '.join(li[1:])
                if name.find('(') != -1:
                    name = name[:name.find('(')-1]
                nb = int(li[0])
                for i in xrange(nb):
                    self.ldeck.append(name)
            lname = []
            for card in self.list_cards:
                lname.append(card.dicinfo['name'])
            for name in self.ldeck:
                if name in lname:
                    continue
                else:
                    self.checkbut.set_label('The deck is NOT valid!\n%s is not in the cards list' % name)
                    fd.close()
                    return False
            self.checkbut.set_label('The deck is valid!')
            fd.close()
            return True

    def choose_deck(self, event=None):
        chooser = gtk.FileChooserDialog(title='Open an existing Deck', action=gtk.FILE_CHOOSER_ACTION_OPEN,
                                        buttons=(gtk.STOCK_CANCEL,gtk.RESPONSE_CANCEL,gtk.STOCK_OPEN,gtk.RESPONSE_OK))
        filter = gtk.FileFilter()
        filter.set_name("All files")
        filter.add_pattern("*");

        chooser.add_filter(filter)
        chooser.show()
        response = chooser.run()
        if response == gtk.RESPONSE_OK:
            self.deck.set_text(chooser.get_filename())
        chooser.destroy()
        return self.check_deck()

if __name__ == '__main__':
    ## main menu
    menu = MainMenu()
    gtk.main()
    ## main menu is closed
    if menu.ok == True:
        ## we launch the Game_Startup
        startup = Game_Startup()
        gtk.main()
        if startup.ok == True:
            g = Game(startup.list_cards, startup.get_deck(startup.deck.get_text()), startup.jabber)
            while True:
                g.main_loop()            

